/* -*- C -*-
 *
 * Copyright 2011 Sandia Corporation. Under the terms of Contract
 * DE-AC04-94AL85000 with Sandia Corporation, the U.S.  Government
 * retains certain rights in this software.
 *
 * Copyright (c) 2016 Intel Corporation. All rights reserved.
 * This software is available to you under the BSD license.
 *
 * This file is part of the Sandia OpenSHMEM software package. For license
 * information, see the LICENSE file in the top level directory of the
 * distribution.
 *
 */

/*
 * This is a generated file, do not edit directly.
 */

#ifndef SHMEMX_H
#define SHMEMX_H

#include <stddef.h>
#include <stdint.h>

#define SHMEMX_THREAD_SINGLE     0
#define SHMEMX_THREAD_FUNNELED   1
#define SHMEMX_THREAD_SERIALIZED 2
#define SHMEMX_THREAD_MULTIPLE   3

#define HAVE_SHMEMX_WTIME

/* Counting puts */
typedef char * shmemx_ct_t;


#ifndef SHMEM_FUNCTION_ATTRIBUTES
#  if OPAL_C_HAVE_VISIBILITY == 1
#     define SHMEM_FUNCTION_ATTRIBUTES __attribute__((visibility("default")))
#  else
#     define SHMEM_FUNCTION_ATTRIBUTES
#  endif
#endif

SHMEM_FUNCTION_ATTRIBUTES void shmemx_init_thread(int tl_requested, int *tl_provided);
SHMEM_FUNCTION_ATTRIBUTES double shmemx_wtime(void);
SHMEM_FUNCTION_ATTRIBUTES char* shmemx_nodename(void);

SHMEM_FUNCTION_ATTRIBUTES void shmemx_getmem_ct(shmemx_ct_t ct, void *target, const void *source, size_t len, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_putmem_ct(shmemx_ct_t ct, void *target, const void *source, size_t len, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_ct_create(shmemx_ct_t *ct);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_ct_free(shmemx_ct_t *ct);
SHMEM_FUNCTION_ATTRIBUTES long shmemx_ct_get(shmemx_ct_t ct);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_ct_set(shmemx_ct_t ct, long value);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_ct_wait(shmemx_ct_t ct, long wait_for);

SHMEM_FUNCTION_ATTRIBUTES void shmemx_pcontrol(int level, ...);

/* AMO: Atomic Bitwise Routines */
SHMEM_FUNCTION_ATTRIBUTES void shmemx_uint_atomic_xor(unsigned int *target, unsigned int value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_ulong_atomic_xor(unsigned long *target, unsigned long value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_ulonglong_atomic_xor(unsigned long long *target, unsigned long long value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_int32_atomic_xor(int32_t *target, int32_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_int64_atomic_xor(int64_t *target, int64_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_uint32_atomic_xor(uint32_t *target, uint32_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_uint64_atomic_xor(uint64_t *target, uint64_t value, int pe);

SHMEM_FUNCTION_ATTRIBUTES void shmemx_uint_atomic_and(unsigned int *target, unsigned int value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_ulong_atomic_and(unsigned long *target, unsigned long value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_ulonglong_atomic_and(unsigned long long *target, unsigned long long value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_int32_atomic_and(int32_t *target, int32_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_int64_atomic_and(int64_t *target, int64_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_uint32_atomic_and(uint32_t *target, uint32_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_uint64_atomic_and(uint64_t *target, uint64_t value, int pe);

SHMEM_FUNCTION_ATTRIBUTES void shmemx_uint_atomic_or(unsigned int *target, unsigned int value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_ulong_atomic_or(unsigned long *target, unsigned long value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_ulonglong_atomic_or(unsigned long long *target, unsigned long long value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_int32_atomic_or(int32_t *target, int32_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_int64_atomic_or(int64_t *target, int64_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_uint32_atomic_or(uint32_t *target, uint32_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES void shmemx_uint64_atomic_or(uint64_t *target, uint64_t value, int pe);

SHMEM_FUNCTION_ATTRIBUTES unsigned int shmemx_uint_atomic_fetch_xor(unsigned int *target, unsigned int value, int pe);
SHMEM_FUNCTION_ATTRIBUTES unsigned long shmemx_ulong_atomic_fetch_xor(unsigned long *target, unsigned long value, int pe);
SHMEM_FUNCTION_ATTRIBUTES unsigned long long shmemx_ulonglong_atomic_fetch_xor(unsigned long long *target, unsigned long long value, int pe);
SHMEM_FUNCTION_ATTRIBUTES int32_t shmemx_int32_atomic_fetch_xor(int32_t *target, int32_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES int64_t shmemx_int64_atomic_fetch_xor(int64_t *target, int64_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES uint32_t shmemx_uint32_atomic_fetch_xor(uint32_t *target, uint32_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES uint64_t shmemx_uint64_atomic_fetch_xor(uint64_t *target, uint64_t value, int pe);

SHMEM_FUNCTION_ATTRIBUTES unsigned int shmemx_uint_atomic_fetch_and(unsigned int *target, unsigned int value, int pe);
SHMEM_FUNCTION_ATTRIBUTES unsigned long shmemx_ulong_atomic_fetch_and(unsigned long *target, unsigned long value, int pe);
SHMEM_FUNCTION_ATTRIBUTES unsigned long long shmemx_ulonglong_atomic_fetch_and(unsigned long long *target, unsigned long long value, int pe);
SHMEM_FUNCTION_ATTRIBUTES int32_t shmemx_int32_atomic_fetch_and(int32_t *target, int32_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES int64_t shmemx_int64_atomic_fetch_and(int64_t *target, int64_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES uint32_t shmemx_uint32_atomic_fetch_and(uint32_t *target, uint32_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES uint64_t shmemx_uint64_atomic_fetch_and(uint64_t *target, uint64_t value, int pe);

SHMEM_FUNCTION_ATTRIBUTES unsigned int shmemx_uint_atomic_fetch_or(unsigned int *target, unsigned int value, int pe);
SHMEM_FUNCTION_ATTRIBUTES unsigned long shmemx_ulong_atomic_fetch_or(unsigned long *target, unsigned long value, int pe);
SHMEM_FUNCTION_ATTRIBUTES unsigned long long shmemx_ulonglong_atomic_fetch_or(unsigned long long *target, unsigned long long value, int pe);
SHMEM_FUNCTION_ATTRIBUTES int32_t shmemx_int32_atomic_fetch_or(int32_t *target, int32_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES int64_t shmemx_int64_atomic_fetch_or(int64_t *target, int64_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES uint32_t shmemx_uint32_atomic_fetch_or(uint32_t *target, uint32_t value, int pe);
SHMEM_FUNCTION_ATTRIBUTES uint64_t shmemx_uint64_atomic_fetch_or(uint64_t *target, uint64_t value, int pe);

#endif /* SHMEMX_H */
